﻿using MS.Win32;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Threading;
using System.Windows.Media.Animation;
using System.Diagnostics;




namespace Calculator
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class MainCalcWnd : Window
    {
    
        const int CornerRadius   = 20;
        CalcResult calcResult = new CalcResult();

        private Point resizePoint = new Point();
        private Size resizeSize = new Size();
        private Point resizeWindowPoint = new Point();
        System.Windows.Threading.DispatcherTimer tmr;

        public MainCalcWnd()
        {
            InitializeComponent();
            resizeSize = new Size(this.Width, this.Height);
            _CurrentDirection = Direction.Drag;   

        }

        enum Direction { Drag, NEConner, North, NWConner, West, SWConner, South, SEConner, East };
        Direction _CurrentDirection;

        private void CloseCalculator(object sender, RoutedEventArgs e)
        {
           Close();
        }
       
        private void AboutCalculator(object sender, RoutedEventArgs e)
        {
            tmr = new System.Windows.Threading.DispatcherTimer();
            tmr.Interval = TimeSpan.FromSeconds(0.1);
            tmr.Tick += TimerOnMinumum;
            tmr.Start();
        }

        void TimerOnMinumum(object sender, EventArgs e)
        {
            Monitor.Enter(this);
            if (Width < 30)
            {
                tmr.Stop();
                Thread.Sleep(1000);
                _Calculator.Visibility = Visibility.Hidden;
                AboutDialog.Visibility = Visibility.Visible;

                tmr = new System.Windows.Threading.DispatcherTimer();
                tmr.Interval = TimeSpan.FromSeconds(0.1);
                tmr.Tick += Restore;
                tmr.Start();
            }
            else
            {
                Width -= 5;
                InvalidateVisual();
            }
            
            Monitor.Exit(this);

        }

        void Restore(object sender, EventArgs e)
        {

            Monitor.Enter(this);

            if (Width >= resizeSize.Width)
            {
                tmr.Stop();
            }
            else
            {
                Width += 5;
                InvalidateVisual();
            }

            Monitor.Exit(this);

        }
                  
            
        private void Window_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            resizePoint = e.GetPosition(this);
            resizeSize = new Size(this.Width, this.Height);
            resizeWindowPoint = new Point(this.Left, this.Top);

            if (NWPath.IsMouseOver)
            {
                _CurrentDirection = Direction.NWConner;
                CaptureMouse();         
            }
            else if (SEPath.IsMouseOver)
            {
                _CurrentDirection = Direction.SEConner;
                CaptureMouse();
            }
            else if (SWPath.IsMouseOver)
            {
                _CurrentDirection = Direction.SWConner;
                CaptureMouse();
            }
            else if (NEPath.IsMouseOver)
            {
                _CurrentDirection = Direction.SEConner;
                CaptureMouse();
            }
            else if (NorthRect.IsMouseOver)
            {
                _CurrentDirection = Direction.North;
                CaptureMouse();
            }
            else if (SouthRect.IsMouseOver)
            {
                _CurrentDirection = Direction.South;
                CaptureMouse();
            }
            else if (EastRect.IsMouseOver)
            {
                _CurrentDirection = Direction.East;
                CaptureMouse();
            }
            else if (WestRect.IsMouseOver)
            {
                _CurrentDirection = Direction.West;
                CaptureMouse();
            }
            else
            {
                _CurrentDirection = Direction.Drag;
                this.DragMove();
            }
            e.Handled = false;

        }

        private void Window_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            if (IsMouseCaptured == true)
            {
                ReleaseMouseCapture();
                resizeSize = new Size(this.Width, this.Height);
            }
        }

        private void Window_MouseMove(object sender, MouseEventArgs e)
        {
            if (IsMouseCaptured == true)
            {
                // UI does not permit us to create new thread.
                // So just use monitor to prevent any thread entering this code.
                Point p = e.GetPosition(this);
                Monitor.Enter(this);
                Point MousePt = Mouse.GetPosition(this);
                double dRatio = 408.0 / 250.0;
                if (_CurrentDirection == Direction.West || _CurrentDirection == Direction.NWConner)
                {
                    double NewWidth = this.resizeSize.Width - (resizePoint.X - p.X);
                    double NewHeight = NewWidth * dRatio;
                    if (NewHeight <= SystemParameters.FullPrimaryScreenHeight)
                    {
                        this.Width = MousePt.X + 20;
                        this.Height = NewHeight;
                        InvalidateVisual();
                    }
                }
                else if (_CurrentDirection == Direction.South || _CurrentDirection == Direction.SWConner)
                {
                    double NewHeight = p.Y + 20;
                    if (NewHeight <= SystemParameters.FullPrimaryScreenHeight)
                    {
                        this.Height = NewHeight;
                        this.Width = NewHeight / dRatio;
                        InvalidateVisual();
                    }
                }
                else if (_CurrentDirection == Direction.East)
                {
                    this.Width = 30;
                    InvalidateVisual();
                }
                
                Monitor.Exit(this);
            }
        }


        private void Button_Click(object sender, RoutedEventArgs e)
        {
            Button btn = (Button)sender;
            switch (btn.Name)
            {
                case "Zero":
                    DisplayTextBox.Text = calcResult.KeyPressed('0');
                    SignDisplay.Text = "";
                    break;
                case "Decimal":
                    DisplayTextBox.Text = calcResult.KeyPressed('.');
                    SignDisplay.Text = "";
                    break;
                case "Equal":
                    DisplayTextBox.Text = calcResult.EqualKeyPressed();
                    SignDisplay.Text = "=";
                    break;
                case "One":
                    DisplayTextBox.Text = calcResult.KeyPressed('1');
                    SignDisplay.Text = "";
                    break;
                case "Two":
                    DisplayTextBox.Text = calcResult.KeyPressed('2');
                    SignDisplay.Text = "";
                    break;
                case "Three":
                    DisplayTextBox.Text = calcResult.KeyPressed('3');
                    SignDisplay.Text = "";
                    break;
                case "Four":
                    DisplayTextBox.Text = calcResult.KeyPressed('4');
                    SignDisplay.Text = "";
                    break;
                case "Five":
                    DisplayTextBox.Text = calcResult.KeyPressed('5');
                    SignDisplay.Text = "";
                    break;
                case "Six":
                    DisplayTextBox.Text = calcResult.KeyPressed('6');
                    SignDisplay.Text = "";
                    break;
                case "Seven":
                    DisplayTextBox.Text = calcResult.KeyPressed('7');
                    SignDisplay.Text = "";
                    break;
                case "Eight":
                    DisplayTextBox.Text = calcResult.KeyPressed('8');
                    SignDisplay.Text = "";
                    break;
                case "Nine":
                    DisplayTextBox.Text = calcResult.KeyPressed('9');
                    SignDisplay.Text = "";
                    break;
                case "PlusBtn":
                    calcResult.Calculate('+');
                    SignDisplay.Text = "+";
                    break;
                case "MinusBtn":
                    calcResult.Calculate('-');
                    SignDisplay.Text = "-";
                    break;
                case "MultiplyBtn":
                    calcResult.Calculate('*');
                    SignDisplay.Text = "*";
                    break;
                case "DivideBtn":
                    calcResult.Calculate('/');
                    SignDisplay.Text = "/";
                    break;
                case "RootBtn":
                    DisplayTextBox.Text = calcResult.SquareRoot();
                    SignDisplay.Text = "";
                    break;
                case "Percentage":
                    DisplayTextBox.Text = calcResult.Percent();
                    SignDisplay.Text = "";
                    break;
                case "ChangeSign":
                    DisplayTextBox.Text = calcResult.ChangeSign();
                    SignDisplay.Text = "";
                    break;
                case "MemoryMRC":
                    DisplayTextBox.Text = calcResult.MemoryMRC();
                    if (calcResult.GetMemory() != 0)
                        MemoryDisplay.Text = "M";
                    else
                        MemoryDisplay.Text = "";
                    break;
                case "MemoryNagative":
                    calcResult.MemoryMinus();
                    if (calcResult.GetMemory() != 0)
                        MemoryDisplay.Text = "M";
                    else
                        MemoryDisplay.Text = "";
                    break;
                case "MemoryPlus":
                    calcResult.MemoryPlus();
                    if (calcResult.GetMemory() != 0)
                        MemoryDisplay.Text = "M";
                    else
                        MemoryDisplay.Text = "";
                    break;
                case "ClearAll":
                    DisplayTextBox.Text = calcResult.Clear();
                    SignDisplay.Text = "";
                    break;
            }
        }

        private void OnHotmailClick(object sender, RoutedEventArgs e)
        {
            Process.Start("mailto:agha.khan@hotmail.com");
        }
        private void OnOkClick(object sender, RoutedEventArgs e)
        {
            _Calculator.Visibility = Visibility.Visible;
            AboutDialog.Visibility = Visibility.Hidden;
        }
    }
}
